#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/4/20'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'


from common import (
    Common
)

from check import (
    Check
)

from convert import (
    Convert
)

from ji_time import (
    JITime
)

from router import (
    Router
)

from security import (
    Security
)

__version__ = "0.25"

__all__ = [
    'Common', 'Check', 'Convert', 'JITime', 'Router', 'Security'
]


#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/4/27'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'

from common import *


class Check(object):

    def __init__(self):
        pass

    @staticmethod
    def previewing(members=None, set_=None):
        """
        :rtype : dict
        :param members: 合法对象的规则描述
        :param set_: 存放对象的集合
        :return: dict格式的检测结果
        """
        if members is None:
            members = []

        if set_ is None:
            set_ = {}

        result = dict()
        result['state'] = Common.exchange_state(20000)

        for item in members:
            if not isinstance(item, tuple):
                result['state'] = Common.exchange_state(41204)
                result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], '，收到 ',
                                                           type(item).__name__, '，源自 ', str(item)])
                return result

            member_type = None
            member_range = None

            if item.__len__() == 3:
                member_type = item[0]
                member_name = item[1]
                member_range = item[2]

            elif item.__len__() == 2:
                member_type = item[0]
                member_name = item[1]

            elif item.__len__() == 1:
                member_name = item[0]

            else:
                result['state'] = Common.exchange_state(41205)
                result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], '，收到 ',
                                                           str(item.__len__()), '，源自 ', str(item)])
                return result

            if type(member_name).__name__ != 'str':
                result['state'] = Common.exchange_state(41207)
                result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], '，收到 ',
                                                           type(member_name).__name__, '，源自 ', str(item)])
                return result

            if member_name not in set_:
                result['state'] = Common.exchange_state(41201)
                result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], str(member_name)])
                return result

            if member_type is not None:
                if not isinstance(set_[member_name], member_type):
                    result['state'] = Common.exchange_state(41202)
                    result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], '，预期类型 ',
                                                               member_type.__name__, '，收到 ',
                                                               type(set_[member_name]).__name__,
                                                               '，源自字段 ', str(member_name)])
                    return result

            if member_range is not None:
                if isinstance(member_range, tuple):
                    if member_range.__len__() < 2:
                        result['state'] = Common.exchange_state(41206)
                        result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], '，预期2个元素，收到 ',
                                                                   str(member_range.__len__()), '，源自 ',
                                                                   str(member_range)])
                        return result

                    if not member_range[0] <= set_[member_name] <= member_range[1]:
                        result['state'] = Common.exchange_state(41203)
                        result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], '，预期取值范围 ',
                                                                   str(member_range), '，收到 ',
                                                                   str(set_[member_name])])
                        return result

                elif isinstance(member_range, list):
                    if member_range.__len__() < 1:
                        result['state'] = Common.exchange_state(41206)
                        result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], '，不可少于1个，收到 ',
                                                                   str(member_range.__len__()), '，源自 ',
                                                                   str(member_range)])
                        return result

                    if set_[member_name] not in member_range:
                        result['state'] = Common.exchange_state(41203)
                        result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], '，预期取值范围 ',
                                                                   str(member_range), '，收到 ',
                                                                   str(set_[member_name])])
                        return result

                else:
                    result['state'] = Common.exchange_state(41206)
                    result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'], '，不支持的类型 ',
                                                               type(member_range).__name__])
                    return result

        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/4/27'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'

import time
import copy
import socket
import os
import hashlib
import random
from state_code import *
import jimit as ji


class Common(object):

    def __init__(self):
        pass

    @staticmethod
    def exchange_state(code):
        """
        :rtype : dict
        :param code: 状态码
        :return: dict格式返回状态信息
        """
        if not isinstance(code, int):
            result = Common.exchange_state(50001)
            return result

        trunk_code = int(code / 100)
        if str(trunk_code) not in index_state['trunk']:
            result = Common.exchange_state(50002)
            return result

        result = copy.copy(index_state['trunk'][(str(trunk_code))])

        if str(code) in index_state['branch']:
            result['sub'] = copy.copy(index_state["branch"][(str(code))])

        return result

    @staticmethod
    def ts():
        """
        :rtype : int
        :return: 当前以秒为单位的时间戳
        """
        return int(time.time())

    @staticmethod
    def tms():
        """
        :rtype : int
        :return: 当前以毫秒为单位的时间戳
        """
        return int(time.time() * 1000)

    @staticmethod
    def tus():
        """
        :rtype : int
        :return: 当前已微妙为单位的时间戳
        """
        return int(time.time() * 1000000)

    @staticmethod
    def get_hostname():
        """
        :rtype : basestring
        :return: 主机名
        """
        return socket.gethostname()

    @staticmethod
    def get_environment(according_to_hostname=True):
        """
        :rtype : str
        :param according_to_hostname: 是否按主机名来识别环境
        :return: 当前所处环境
        """

        def exchange_env(environment_string):
            if environment_string.lower().find('debug') != -1:
                return 'debug'
            elif environment_string.lower().find('sandbox') != -1:
                return 'sandbox'
            else:
                return 'production'

        if according_to_hostname:
            environment = exchange_env(Common.get_hostname())
        else:
            environment = exchange_env(os.environ.get('JI_ENVIRONMENT', ''))

        return environment

    @staticmethod
    def calc_sha1_by_file(file_path):
        """
        :rtype : dict
        :param file_path: 欲计算其sha1 hash值的文件路径
        :return: lower格式的sha1 hash值
        """
        result = dict()
        result['state'] = Common.exchange_state(20000)

        if not os.path.isfile(file_path):
            result['state'] = Common.exchange_state(40401)
            result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'],
                                                       '，目标"', file_path, '"不是一个有效文件'])
            return result

        with open(file_path, 'rb') as f:
            try:
                sha1_obj = hashlib.sha1()
                sha1_obj.update(f.read())
                result['sha1'] = sha1_obj.hexdigest()
            except Exception, e:
                result['state'] = Common.exchange_state(50004)
                result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'],
                                                           '，详细信息: ', e.message])
                return result

        return result

    @staticmethod
    def calc_sha1_by_fd(fd):
        """
        :rtype : dict
        :param fd: 欲计算其sha1 hash值的文件描述符
        :return: lower格式的sha1 hash值
        """
        result = dict()
        result['state'] = Common.exchange_state(20000)

        try:
            sha1_obj = hashlib.sha1()
            sha1_obj.update(fd.read())
            result['sha1'] = sha1_obj.hexdigest()
        except Exception, e:
            result['state'] = Common.exchange_state(50004)
            result['state']['sub']['zh-cn'] = ''.join([result['state']['sub']['zh-cn'],
                                                       '，详细信息: ', e.message])
            return result
        finally:
            fd.seek(0, 0)

        return result

    @staticmethod
    def generate_random_code(length, letter_form='mix', numeral=True, spechars=False):
        """
        :rtype : dict
        :param length: 随机值长度，最长1000字符
        :param letter_form: 字母表现形式['lower'(小写) | 'upper'(大写) | 'mix'(混合)]
        :param numeral: 是否允许数字参选随机值
        :param spechars: 是否允许特殊字符参选随机值
        :return: 随机选取的字符串
        """
        args_rules = [
            (int, 'length', (1, 1000)),
            (basestring, 'letter_form', ['lower', 'upper', 'mix']),
            (bool, 'numeral'),
            (bool, 'spechars')
        ]

        ret = ji.Check.previewing(args_rules, locals())
        if '200' != ret['state']['code']:
            return ret

        result = dict()
        result['state'] = Common.exchange_state(20000)

        upper = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
                 'U', 'V', 'W', 'X', 'Y', 'Z']
        lower = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
                 'u', 'v', 'w', 'x', 'y', 'z']
        number = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']
        special_characters = ['!', '"', '#', '$', '%', '&', '\'', '(', ')', '*', '+', ',', '-', '.', '/', ':', ';', '<',
                              '=', '>', '?', '@', '[', '\\', ']', '^', '_', '`', '{', '|', '}', '~']

        character_codes = []
        if letter_form == 'lower':
            character_codes.extend(lower)
        elif letter_form == 'upper':
            character_codes.extend(upper)
        elif letter_form == 'mix':
            character_codes.extend(lower)
            character_codes.extend(upper)

        if numeral:
            character_codes.extend(number)

        if spechars:
            character_codes.extend(special_characters)

        result['random_code'] = ''

        while length:
            length -= 1
            result['random_code'] = ''.join([result['random_code'], random.choice(character_codes)])

        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/5/16'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'

from common import *
import datetime


class JITime(object):

    def __init__(self):
        pass

    @staticmethod
    def gmt(ts=Common.ts()):
        """
        :rtype : str
        :param ts: 指定时间戳，默认系统当前时间戳
        :return: GMT格式的当前时间('Thu, 18 Jun 2015 10:06:28 GMT')
        """
        return time.strftime('%a, %d %b %Y %H:%M:%S GMT', time.gmtime(ts))

    @staticmethod
    def today(separator='-'):
        """
        :rtype : str
        :param separator: 年、月、日之间的间隔符，默认为'-'
        :return: 当天日期('2015-06-18')
        """
        fmt = separator.join(['%Y', '%m', '%d'])
        return time.strftime(fmt)

    @staticmethod
    def now_time(separator=':'):
        """
        :rtype : str
        :param separator: 时、分、秒之间的间隔符，默认为':'
        :return: 当前系统的时间('18:08:28')
        """
        fmt = separator.join(['%H', '%M', '%S'])
        return time.strftime(fmt)

    @staticmethod
    def now_date_time(date_separator='-', time_separator=':', dt_separator=' '):
        """
        :rtype : str
        :param date_separator: 日期年、月、日之间的间隔符，默认为'-'
        :param time_separator: 时间时、分、秒之间的间隔符，默认为':'
        :param dt_separator: 日期与时间之间的间隔符，默认为' '
        :return: 当前系统的日期时间('2015-06-18 18:12:04')
        """
        fmt = dt_separator.join([date_separator.join(['%Y', '%m', '%d']), time_separator.join(['%H', '%M', '%S'])])
        return time.strftime(fmt)

    @staticmethod
    def the_day(the_day, separator='-'):
        fmt = separator.join(['%Y', '%m', '%d'])

        if not isinstance(the_day, datetime.date):
            raise TypeError('only datetime.date')

        return the_day.strftime(fmt)

    @staticmethod
    def the_day_before_today(offset=0, separator='-'):
        """
        :rtype : str
        :param offset: 以日为单位，向前偏移n值的日期
        :param separator: 日期年、月、日之间的分隔符，默认为'-'
        :return: 向前偏移n日的日期(jimit.JITime.the_day_before_today(20) --> '2015-05-29')
        """
        return JITime.the_day(datetime.date.today()-datetime.timedelta(days=offset), separator)

    @staticmethod
    def the_day_after_today(offset=0, separator='-'):
        """
        :rtype : str
        :param offset: 以日为单位，向后偏移n值的日期
        :param separator: 日期年、月、日之间的间隔符，默认值'-'
        :return: 向后偏移n日的日期(jimit.JITime.the_day_after_today(20) --> '2015-07-08')
        """
        return JITime.the_day(datetime.date.today()+datetime.timedelta(days=offset), separator)

    @staticmethod
    def week():
        """
        :rtype : str
        :return: 当前系统时间所在当年的周期('24')
        """
        return time.strftime('%W')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/4/27'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'

import decimal

from common import *


class Convert(object):

    def __init__(self):
        pass

    @staticmethod
    def sql2json(sql_obj):
        j_sql = dict()
        result = dict()
        result['state'] = Common.exchange_state(20000)

        try:
            for col in sql_obj._sa_class_manager.mapper.mapped_table.columns:
                if isinstance(getattr(sql_obj, col.name), decimal.Decimal):
                    j_sql[col.name] = str(getattr(sql_obj, col.name))
                else:
                    j_sql[col.name] = getattr(sql_obj, col.name)
        except Exception, e:
            result['state'] = Common.exchange_state(50003)
            result['state']['sub']['detail'] = e.message
            return result

        result['j_sql'] = j_sql
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/6/24'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'

from common import *
import hashlib

class Security(object):

    @staticmethod
    def ji_pbkdf2(password='', quality=1000, algorithm='sha1', salt=''):
        """ jimit pbkdf2的实现
        :rtype: dict
        :param password: 输入的密码
        :param quality: 质量(复杂度，即循环执行散列算法多少次，默认1000)
        :param algorithm: 散列算法，支持的有['md5', 'sha1', 'sha256', 'sha512']，默认是sha1
        :param salt: 可以自定义salt，如果不指定，将产生默认32字符的salt，该参数主要用于ji_pbkdf2_check
        :return: 返回字典中password_hash为生成的秘密
        """
        password_length = password.__len__()
        args_rules = [
            (basestring, 'password'),
            (int, 'quality', (1, 10000)),
            (basestring, 'algorithm', ['md5', 'sha1', 'sha256', 'sha512']),
            (int, 'password_length', (8, 100)),
            (basestring, 'salt')
        ]

        ret = ji.Check.previewing(args_rules, locals())
        if '200' != ret['state']['code']:
            return ret

        if salt == '':
            ret = Common.generate_random_code(length=32)
            if '200' != ret['state']['code']:
                return ret

            salt = ret['random_code']
            del ret['random_code']
        else:
            salt = salt

        password_hash = ''.join([salt, password])
        hash_method = None
        if algorithm == 'md5':
            hash_method = hashlib.md5
        elif algorithm == 'sha1':
            hash_method = hashlib.sha1
        elif algorithm == 'sha256':
            hash_method = hashlib.sha256
        elif algorithm == 'sha512':
            hash_method = hashlib.sha512
        else:
            raise KeyError(''.join(['Not support algorithm: ', algorithm]))

        tmp_quality = quality
        while tmp_quality:
            password_hash = hash_method(password_hash).hexdigest()
            tmp_quality -= 1

        ret['password_hash'] = '$'.join(['ji_pbkdf2', algorithm, str(quality), salt, password_hash])
        return ret

    @staticmethod
    def ji_pbkdf2_check(password='', password_hash=''):
        """ 校验密码
        :rtype : dict
        :param password: 原始密码
        :param password_hash: 比对的密码hash字符串
        :return: 返回的字典中没有auth_pass key或auth_pass为False都为校验失败，只有auth_pass为True时才为校验通过
        """
        password_hash_segment = password_hash.split('$')
        password_hash_segment_length = password_hash_segment.__len__()
        args_rules = [
            (basestring, 'password'),
            (basestring, 'password_hash'),
            (int, 'password_hash_segment_length', [5])
        ]

        ret = ji.Check.previewing(args_rules, locals())
        if '200' != ret['state']['code']:
            return ret

        method = password_hash_segment[0]
        algorithm = password_hash_segment[1]
        quality = int(password_hash_segment[2])
        salt = password_hash_segment[3]

        ret = Security.ji_pbkdf2(password=password, quality=quality, algorithm=algorithm, salt=salt)
        if '200' != ret['state']['code']:
            return ret

        if ret['password_hash'] == password_hash:
            ret['auth_pass'] = True

        del ret['password_hash']
        return ret



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/5/16'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'

from jimit import Common

router_table = {}


class Router(object):

    def __init__(self):
        pass

    @staticmethod
    def not_found(content):
        return Common.exchange_state(50100)

    @staticmethod
    def launcher(**kwargs):
        action = kwargs.get('action', None)
        content = kwargs.get('content', None)
        return router_table.get(action, Router.not_found)(content)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'James Iter'
__date__ = '15/4/20'
__contact__ = 'james.iter.cn@gmail.com'
__copyright__ = '(c) 2015 by James Iter.'

import codecs
import os

from setuptools import setup, find_packages


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()


PACKAGE = "jimit"
NAME = "jimit"
DESCRIPTION = "James Iter's common library by python."
AUTHOR = "James Iter"
AUTHOR_EMAIL = "james.iter.cn@gmail.com"
URL = "https://github.com/jamesiter/jimitlib-py.git"
VERSION = __import__(PACKAGE).__version__

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=read("README.rst"),
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license="BSD",
    url=URL,
    packages=find_packages(exclude=["tests.*", "tests"]),
    package_data={
        '': ['*.json'],
    },
    classifiers=[
        "Development Status :: 1 - Planning",
        "Environment :: Plugins",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        ],
    zip_safe=False,
)